/*
 * xtitle.c
 * set window title for xterm/jterm/kterm/decterm/cmdtool
 *
 * Kazuro Furukawa
 * High Energy Accelerator Research Organization (KEK), Japan
 * kazuro.furukawa@kek.jp
 *
 * originally written by A.Shirahashi, KEK
 *    for vms, This program is placed in public domain.
 *
 * modified by K.Furukawa, KEK, on Jun.4.1991.
 *    for unix (sunos,ultrix,next,hpux,sony,sxa,umx,...) and vms+win
 * modified by K.Furukawa, KEK, on Sep.7.1991.
 *    for decterm as well as xterm/kterm
 * modified by K.Furukawa, KEK, on May.23.1992. (v.2.4)
 *    put ':' to host, '<>' to user
 * modified by K.Furukawa, KEK, on Apr.12.1993. (v.2.5)
 *    ifdef for sony News-OS
 * modified by K.Furukawa, KEK, on Dec.22.1993. (v.2.6)
 *    remove domain name part from hostname
 * modified by G.Denkinson, University of Nottigham, on Feb.25.1994. (v.2.4.1)
 *    added cmdtool support
 * modified by K.F, Apr.18.1994. (v.2.7)
 *    merge modification by G.Denkinson, add ifdef for lynx and os9
 * modified by K.Furukawa, KEK, on Jan.20.2004. (v.2.8)
 *    use getpwuid(geteuid()) instead of cuserid() for portability
 *
 * usage:
 *   xtitle [options] [title]
 * options:
 *   -x: xterm/kterm(default)          -t: decterm (dxterm)
 *   -c: sun cmdtool
 *   -h: hostname(default)             -u: user name
 *   -p: current host and directory    -d: add disk name
 *   -0: title and icon    -1: icon    -2: title
 * example:
 *   xtitle Window 1-A
 *   xtitle -pd
 *   xtitle -uh
 *   xtitle -t Decterm A
 *   xtitle
 * compile:
 *   cc -o xtitle xtitle.c
 *   (unneccessary to link with X11)
 *   (on vms, link with tcp/ip library such as WIN for gethostname() )
 *   (it runs on sunos,ultrix,next,hp-ux,sony,sxa,umx,linux,freebsd,
 *    osf1/digitalunix/tru64unix,hi-ux,aix,bsdos,darwin,lynxos,cygwin,etc.)
 * bug:
 *   it should automatically check the type of terminal emulators, xterm
 *   or decterm.  if xtitle is invoked on decterm without "-t" option,
 *   decterm locks (decterm must be reset, sorry).
 */

char xtitle_what[] = 
"@(#)xtitle v2.8 set title for xterm/kterm/decterm/cmdtool";
char xtitle_what2[] = 
"@(#) written by a.shirahashi, k.furukawa, g.denkinson";
char xtitle_what3[] = 
"@(#) kazuro.furukawa@kek.jp, Jun.1991-Jan.2004.";

#include <stdio.h>
#ifndef OSK
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

main(argc,argv)
    int argc;
    char **argv;
{
    int option = 0, numarg = 0;
    char c, title[256], t[256], *p;
    int hopt = 0, popt = 0, dopt = 0, uopt = 0, topt = 0;

    title[0] = '\0';

/* analyze command line arguments */

    while( --argc ) {
	if( **(++argv) == '-' ) {
	    while( c = *(++(*argv)) ) {
		switch( c ) {
		case '0':		/* Title and Icon */
		    option = 0; break;
		case '1':		/* Icon */ 
		    option = 1; break;
		case '2':		/* Title */ 
		    option = 2; break;
		case 'h':
		    hopt = 1; break;
		case 'p':
		    popt = 1; break;
		case 'd':
		    dopt = 1; break;
		case 'u':
		    uopt = 1; break;
		case 'x':
		    topt = 0; break;
		case 't':
		    topt = 1; break;
		case 'c':
		    topt = 2; break;
		default: 
		    fprintf(stderr,"xtitle: no such option: %c\n",c);
		    fprintf(stderr,"usage:  xtitle [options] [title]\n");
		    fprintf(stderr,"options:\n");
	fprintf(stderr,"  -x: xterm/kterm(default)       -t: decterm\n");
	fprintf(stderr,"  -c: sun cmdtool\n");
        fprintf(stderr,"  -h: hostname(default)          -u: user name\n");
        fprintf(stderr,"  -p: current host & directory   -d: add disk name\n");
        fprintf(stderr,"  -0: title & icon    -1: icon   -2: title\n");
        fprintf(stderr,"example:  xtitle -uh\n");
		    exit(1);
		}
	    }
	} else {
	    if (numarg++) strcat( title," " );
	    strcat( title,*argv );
	}
    }

    if( numarg*(hopt+uopt+popt) ) strcat( title," " );

/* user (login) name */

    if( uopt ) {
	if( !hopt ) {
	    strcat( title,"<" );
	}
#if 0
#ifndef NeXT
#ifndef sony
	strcat( title,cuserid(NULL) );
#else
	strcat( title,getlogin() );
#endif
#else
	strcat( title,getlogin() );
#endif
#endif

#ifdef OSK
	strcat( title,cuserid(NULL) );
#else /* OSK */
	{
	    struct passwd *pwuid = getpwuid(geteuid());
	    strcat( title,pwuid->pw_name );
	}
#endif /* OSK */

	if( hopt ) {
	    strcat( title,"@" );
	} else {
	    strcat( title,">" );
	}
    }

/* host name */

    if( title[0] == '\0' || hopt ) {
	gethostname(t,256);
	if( p = strchr(t,'.') )  *p = '\0';	/* remove domain name */
	strcat( title,t );
	if( popt ) {
	    strcat( title,":" );
	}
    }
  
/* current working directory */

    if( popt ) {
#ifndef NeXT
#ifndef Lynx
	getcwd(t,256);
#else
	getwd(t);			/* on lynx getcwd is in libc_p.a */
#endif
#else
	getwd(t);
#endif
	if( dopt ) {				/* for vms */
	    strcat( title,t );
	} else {
	    if( p = strchr( t,':' ) ) {
		strcat( title,p+1 );
	    } else {
		strcat( title,t );
	    }
	} 
    }

    p = title + strlen(title);
    if( *(p-1) == ':' && *(p-2) == ':' ) {
	*(p-2) = '\0';
    }

    switch( topt )  {
    case 0:				/* xterm/kterm */
#ifndef ebcdic
	printf("\033]%d;%s\007",option,title);
#else
	printf("\047]%d;%s\057",option,title);
#endif
	break;

    case 1:				/* decterm */
#ifndef ebcdic
	if (!(option&2))  printf("\033]2L;%s\033\\",title);
	if (!(option&1))  printf("\033]21;%s\033\\",title);
#else
	if (!(option&2))  printf("\047]2L;%s\047\\",title);
	if (!(option&1))  printf("\047]21;%s\047\\",title);
#endif
	break;

    case 2:				/* cmdtool */
#ifndef ebcdic
        if (!(option&2))  printf("\033]L%s\033\\",title);
        if (!(option&1))  printf("\033]l%s\033\\",title);
#else
        if (!(option&2))  printf("\047]L%s\047\\",title);
        if (!(option&1))  printf("\047]l%s\047\\",title);
#endif
    }
	
}
